/*
 * Copyright 2010 Mihai Niculescu <q.quark@gmail.com>
 *
 * This file is part of EqualX Project (https://launchpad.net/equalx/)
 *
 * EqualX is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * EqualX is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "latexeditor.h"
#include "latexhighlighter.h"

#include <QAbstractItemView>
#include <QCompleter>
#include <QFile>
#include <QKeyEvent>
#include <QStringListModel>
#include <QScrollBar>

LatexEditor::LatexEditor(QWidget *parent)
    : QTextEdit(parent),
    c(0),
    minCompletionChars(2)
{
    setWindowTitle(tr("Latex Widget Editor"));

    c = new QCompleter(this);
    c->setModel( modelFromFile(":/resources/templates/latexKeyWords.txt") );
    c->setCaseSensitivity(Qt::CaseInsensitive);
    c->setWrapAround(false);

    highlighter = new LatexHighlighter( this->document() );
    setCompleter(c);
}

LatexEditor::~LatexEditor()
{

}


QAbstractItemModel *LatexEditor::modelFromFile(const QString& fileName)
{
    QFile file(fileName);
    if(!file.open(QFile::ReadOnly) )
        return new QStringListModel(c);

    QStringList words;

    while (!file.atEnd()) {
        QByteArray line = file.readLine();
        if (!line.isEmpty())
            words << line.trimmed();
    }

    return new QStringListModel(words, c);

}

void LatexEditor::setCompleter(QCompleter *completer)
{
    if(c)
        QObject::disconnect(c, 0, this, 0);

    c = completer;

    if(!c)
        return;

    c->setWidget(this);
    c->setCompletionMode(QCompleter::PopupCompletion);
    c->setCaseSensitivity(Qt::CaseInsensitive);
    QObject::connect(c, SIGNAL(activated(const QString&)),
                     this, SLOT(insertCompletion(QString)));
}

QCompleter *LatexEditor::completer() const
{
    return c;
}

void LatexEditor::insertCompletion(const QString &completion)
{
    if(c->widget() != this)
        return;
    QTextCursor tc = textCursor();
    int extra = completion.length() - c->completionPrefix().length();
    tc.movePosition(QTextCursor::Left);
    tc.movePosition(QTextCursor::EndOfWord);
    tc.insertText(completion.right(extra));
    setTextCursor(tc);
}

QString LatexEditor::textUnderCursor() const
{
    QTextCursor tc = textCursor();
    tc.select(QTextCursor::WordUnderCursor );
//QString::SectionFlag flag =  QString::SectionSkipEmpty | QString::SectionIncludeLeadingSep;

    QString selectedText = tc.selectedText();
//    int i = selectedText.lastIndexOf("\\");

    return selectedText.section("\\", -1,-1, QString::SectionSkipEmpty | QString::SectionIncludeLeadingSep);
}

void LatexEditor::focusInEvent(QFocusEvent *e)
{
    if(c)
        c->setWidget(this);
    QTextEdit::focusInEvent(e);
}

/* The keyPressEvent() is reimplemented to ignore key events like
 Qt::Key_Enter, Qt::Key_Return, Qt::Key_Escape, Qt::Key_Tab, and Qt::Key_Backtab
 so the completer can handle them.

If there is an active completer, we cannot process the shortcut, Ctrl+E.*/
void LatexEditor::keyPressEvent(QKeyEvent *e)
{
    if(c && c->popup()->isVisible()) {
        switch(e->key()) {
        case Qt::Key_Enter:
        case Qt::Key_Return:
        case Qt::Key_Escape:
        case Qt::Key_Tab:
        case Qt::Key_Backtab:
            e->ignore();
            return;

        default:
            break;

        }
    }

    bool isShortcut = ((e->modifiers() & Qt::ControlModifier) && e->key() == Qt::Key_E); // CTRL+E
    if(!c || !isShortcut)
        QTextEdit::keyPressEvent(e);

    /*  We also handle other modifiers and shortcuts
    for which we do not want the completer to respond to.*/

    const bool ctrlOrShift = e->modifiers() & (Qt::ControlModifier | Qt::ShiftModifier);
    if (!c || (ctrlOrShift && e->text().isEmpty()))
        return;

    static QString eow("~!@#$%&*()+|:\"<>?,./;'[]-="); // end of word
    bool hasModifier = (e->modifiers() != Qt::NoModifier) && !ctrlOrShift;
    QString completionPrefix = textUnderCursor();

    if (!isShortcut && (hasModifier || e->text().isEmpty() || completionPrefix.length() < minCompletionChars
                        || eow.contains(e->text().right(1)) ) ) {
        c->popup()->hide();
        return;
    }

    if (completionPrefix != c->completionPrefix()) {
        c->setCompletionPrefix(completionPrefix);
        c->popup()->setCurrentIndex(c->completionModel()->index(0, 0));
    }

    QRect cr = cursorRect();
    cr.setWidth(c->popup()->sizeHintForColumn(0)
                + c->popup()->verticalScrollBar()->sizeHint().width());
    c->complete(cr); // popup it up!

}
