/*
 * Copyright 2010 Mihai Niculescu <q.quark@gmail.com>
 *
 * This file is part of EqualX Project (https://launchpad.net/equalx/)
 *
 * EqualX is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * EqualX is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <QAction>
#include <QCloseEvent>
#include <QDir>
#include <QFile>
#include <QFileDialog>

#include <QGraphicsScene>
#include <QGraphicsPixmapItem>
#include <QGridLayout>
#include <QLayout>
#include <QMenu>
#include <QPixmap>
#include <QProcess>
#include <QPushButton>
#include <QSettings>
#include <QTextStream>
#include <QToolButton>
#include <QMessageBox>


#include <QDebug>
#include "defines.h"
#include "EquationPixmapItem.h"
#include "latexeditor.h"
#include "WidgetToolMenu.h"

#include "mainwindow.h"
#include "ui_mainwindow.h"

MainWindow::MainWindow(QWidget *parent)
        : QMainWindow(parent),
        ui(new Ui::MainWindowClass),
        equation(0),
        timer(this),
        resolution(150),
        fgColor("black"),
        bgColor("Transparent"),
        isInline(false),
        isDirty(false), nUpdates(0)

{
    ui->setupUi(this);

    setWindowTitle(APP_FULL_NAME);

    scene = new QGraphicsScene();

    editor = ui->editEquation;

    ui->View->setScene(scene);
    ui->fgColorPicker->setCurrentIndex(2);
    ui->bgColorPicker->setCurrentIndex(1);
    ui->groupBoxProperties->setVisible(false);
    ui->groupBoxSymbols->setVisible(false);

    equation = new EquationPixmapItem;
    equation->setVisible(true);
    equation->setAcceptDrops(true);
    equation->setToolTip("Drag and Drop this");

    scene->addItem(equation);

    readSettings();

    setActions();
    setMathPanel();
    setMathPanelTabsActions();
    setSignals();

    setupParser();
    loadTexTemplate();

    setCurrentFile("");
    setUnifiedTitleAndToolBarOnMac(true);
    ui->fontSizeComboBox->setCurrentIndex(0);

}

MainWindow::~MainWindow()
{
    delete ui;
}

void MainWindow::setActions()
{
    /* Menus for Mathbar */
    toolDeco = new WidgetToolMenu(2, 1, this);
    toolDeco->addAction(ui->actionDeco_super);
    toolDeco->addAction(ui->actionDeco_under);
    toolDeco->addAction(ui->actionDeco_overset);
    toolDeco->addAction(ui->actionDeco_underset);

    toolFunc= new WidgetToolMenu(2, 1, this);
    toolFunc->addAction(ui->actionFunc_frac);
    toolFunc->addAction(ui->actionFunc_root);
    toolFunc->addAction(ui->actionFunc_fracNoHLine);
    toolFunc->addAction(ui->actionFunc_sqrt);

    toolDelim= new WidgetToolMenu(3, 1, this);
    toolDelim->addAction(ui->actionDelim_paraSimple);
    toolDelim->addAction(ui->actionDelim_squareSimple);
    toolDelim->addAction(ui->actionDelim_ceilSimple);
    toolDelim->addAction(ui->actionDelim_curlySimple);
    toolDelim->addAction(ui->actionDelim_angleSimple);
    toolDelim->addAction(ui->actionDelim_floorSimple);
    toolDelim->addAction(ui->actionDelim_vertSimple);
    toolDelim->addAction(ui->actionDelim_barsSimple);


    toolAccents = new WidgetToolMenu(3, 1, this);
    toolAccents->addAction(ui->actionAccents_hat);
    toolAccents->addAction(ui->actionAccents_acute);
    toolAccents->addAction(ui->actionAccents_breve);
    toolAccents->addAction(ui->actionAccents_tilde);
    toolAccents->addAction(ui->actionAccents_underleftrightarrow);
    toolAccents->addAction(ui->actionAccents_underline);
    toolAccents->addAction(ui->actionAccents_bar);
    toolAccents->addAction(ui->actionAccents_check);
    toolAccents->addAction(ui->actionAccents_dot);
    toolAccents->addAction(ui->actionAccents_dotdouble);
    toolAccents->addAction(ui->actionAccents_grave);
    toolAccents->addAction(ui->actionAccents_hatwide);
    toolAccents->addAction(ui->actionAccents_overbrace);
    toolAccents->addAction(ui->actionAccents_overleftarrow);
    toolAccents->addAction(ui->actionAccents_overleftrightarrow);
    toolAccents->addAction(ui->actionAccents_overrightarrow);
    toolAccents->addAction(ui->actionAccents_overline);
    toolAccents->addAction(ui->actionAccents_tildewide);
    toolAccents->addAction(ui->actionAccents_underbrace);
    toolAccents->addAction(ui->actionAccents_underleftarrow);
    toolAccents->addAction(ui->actionAccents_underrightarrow);
    toolAccents->addAction(ui->actionAccents_vector);

    toolOps = new WidgetToolMenu( 3, 1, this);
    toolOps->addAction(ui->actionOps_pm);
    toolOps->addAction(ui->actionOps_times);
    toolOps->addAction(ui->actionOps_div);
    toolOps->addAction(ui->actionOps_mp);
    toolOps->addAction(ui->actionOps_cap);
    toolOps->addAction(ui->actionOps_cup);
    toolOps->addAction(ui->actionOps_sqcap);
    toolOps->addAction(ui->actionOps_sqcup);
    toolOps->addAction(ui->actionOps_uplus);
    toolOps->addAction(ui->actionOps_amalg);
    toolOps->addAction(ui->actionOps_ominus);
    toolOps->addAction(ui->actionOps_oplus);
    toolOps->addAction(ui->actionOps_otimes);
    toolOps->addAction(ui->actionOps_oslash);
    toolOps->addAction(ui->actionOps_odot);
    toolOps->addAction(ui->actionOps_cdot);
    toolOps->addAction(ui->actionOps_circ);
    toolOps->addAction(ui->actionOps_bullet);
    toolOps->addAction(ui->actionOps_bigcirc);
    toolOps->addAction(ui->actionOps_star);
    toolOps->addAction(ui->actionOps_diamond);
    toolOps->addAction(ui->actionOps_vee);
    toolOps->addAction(ui->actionOps_wedge);
    toolOps->addAction(ui->actionOps_setminus);
    toolOps->addAction(ui->actionOps_wr);
    toolOps->addAction(ui->actionOps_dagger);
    toolOps->addAction(ui->actionOps_daggerd);
    toolOps->addAction(ui->actionOps_triangleleft);
    toolOps->addAction(ui->actionOps_triangleright);
    toolOps->addAction(ui->actionOps_bigtriangleup);
    toolOps->addAction(ui->actionOps_bigtriangledown);

    toolRelations = new WidgetToolMenu( 3, 1, this);
    toolRelations->addAction(ui->actionRel_leq);
    toolRelations->addAction(ui->actionRel_neq);
    toolRelations->addAction(ui->actionRel_simeq);
    toolRelations->addAction(ui->actionRel_geq);
    toolRelations->addAction(ui->actionRel_equiv);
    toolRelations->addAction(ui->actionRel_models);
    toolRelations->addAction(ui->actionRel_prec);
    toolRelations->addAction(ui->actionRel_succ);
    toolRelations->addAction(ui->actionRel_sim);
    toolRelations->addAction(ui->actionRel_perp);
    toolRelations->addAction(ui->actionRel_preceq);
    toolRelations->addAction(ui->actionRel_succeq);
    toolRelations->addAction(ui->actionRel_mid);
    toolRelations->addAction(ui->actionRel_lless);
    toolRelations->addAction(ui->actionRel_ggreater);
    toolRelations->addAction(ui->actionRel_asymp);
    toolRelations->addAction(ui->actionRel_parallel);
    toolRelations->addAction(ui->actionRel_subset);
    toolRelations->addAction(ui->actionRel_supset);
    toolRelations->addAction(ui->actionRel_approx);
    toolRelations->addAction(ui->actionRel_smile);
    toolRelations->addAction(ui->actionRel_subseteq);
    toolRelations->addAction(ui->actionRel_supseteq);
    toolRelations->addAction(ui->actionRel_cong);
    toolRelations->addAction(ui->actionRel_frown);
    toolRelations->addAction(ui->actionRel_sqsubseteq);
    toolRelations->addAction(ui->actionRel_sqsupseteq);
    toolRelations->addAction(ui->actionRel_dotequal);
    toolRelations->addAction(ui->actionRel_in);
    toolRelations->addAction(ui->actionRel_ni);
    toolRelations->addAction(ui->actionRel_propto);
    toolRelations->addAction(ui->actionRel_notin);
    toolRelations->addAction(ui->actionRel_vdash);
    toolRelations->addAction(ui->actionRel_dashv);
    toolRelations->addAction(ui->actionRel_bowtie);

    toolMisc = new WidgetToolMenu(3, 1, this);
    toolMisc->addAction(ui->actionMisc_infty);
    toolMisc->addAction(ui->actionMisc_forall);
    toolMisc->addAction(ui->actionMisc_exists);
    toolMisc->addAction(ui->actionMisc_nabla);
    toolMisc->addAction(ui->actionMisc_partial);
    toolMisc->addAction(ui->actionMisc_prime);
    toolMisc->addAction(ui->actionMisc_ell);
    toolMisc->addAction(ui->actionMisc_emptyset);
    toolMisc->addAction(ui->actionMisc_imath);
    toolMisc->addAction(ui->actionMisc_jmath);
    toolMisc->addAction(ui->actionMisc_wp);
    toolMisc->addAction(ui->actionMisc_textrmO);
    toolMisc->addAction(ui->actionMisc_hbar);

    toolSets = new WidgetToolMenu(3,1, this);
    toolSets->addAction(ui->actionSets_re);
    toolSets->addAction(ui->actionSets_mathbbZ);
    toolSets->addAction(ui->actionSets_mathcalH);
    toolSets->addAction(ui->actionSets_im);
    toolSets->addAction(ui->actionSets_mathbbC);
    toolSets->addAction(ui->actionSets_mathcalF);
    toolSets->addAction(ui->actionSets_aleph);
    toolSets->addAction(ui->actionSets_mathbbH);
    toolSets->addAction(ui->actionSets_mathcalL);
    toolSets->addAction(ui->actionSets_mathbbN);
    toolSets->addAction(ui->actionSets_mathbbQ);
    toolSets->addAction(ui->actionSets_mathcalO);
    toolSets->addAction(ui->actionSets_mathbbR);

    toolGreek = new WidgetToolMenu(3,1 , this);
    toolGreek->addAction(ui->actionGreek_alpha);
    toolGreek->addAction(ui->actionGreek_beta);
    toolGreek->addAction(ui->actionGreek_chi);
    toolGreek->addAction(ui->actionGreek_delta);
    toolGreek->addAction(ui->actionGreek_epsilon);
    toolGreek->addAction(ui->actionGreek_epsilonVar);
    toolGreek->addAction(ui->actionGreek_eta);
    toolGreek->addAction(ui->actionGreek_gamma);
    toolGreek->addAction(ui->actionGreek_iota);
    toolGreek->addAction(ui->actionGreek_kappa);
    toolGreek->addAction(ui->actionGreek_lambda);
    toolGreek->addAction(ui->actionGreek_mu);
    toolGreek->addAction(ui->actionGreek_nu);
    toolGreek->addAction(ui->actionGreek_omega);
    toolGreek->addAction(ui->actionGreek_phi);
    toolGreek->addAction(ui->actionGreek_phiVar);
    toolGreek->addAction(ui->actionGreek_pi);
    toolGreek->addAction(ui->actionGreek_piVar);
    toolGreek->addAction(ui->actionGreek_psi);
    toolGreek->addAction(ui->actionGreek_rho);
    toolGreek->addAction(ui->actionGreek_rhoVar);
    toolGreek->addAction(ui->actionGreek_sigma);
    toolGreek->addAction(ui->actionGreek_sigmaVar);
    toolGreek->addAction(ui->actionGreek_tau);
    toolGreek->addAction(ui->actionGreek_theta);
    toolGreek->addAction(ui->actionGreek_thetaVar);
    toolGreek->addAction(ui->actionGreek_upsilon);
    toolGreek->addAction(ui->actionGreek_xi);
    toolGreek->addAction(ui->actionGreek_zeta);

    toolGreekBig = new WidgetToolMenu(3, 1, this);
    toolGreekBig->addAction(ui->actionGreekBig_delta);
    toolGreekBig->addAction(ui->actionGreekBig_gamma);
    toolGreekBig->addAction(ui->actionGreekBig_lambda);
    toolGreekBig->addAction(ui->actionGreekBig_phi);
    toolGreekBig->addAction(ui->actionGreekBig_pi);
    toolGreekBig->addAction(ui->actionGreekBig_psi);
    toolGreekBig->addAction(ui->actionGreekBig_sigma);
    toolGreekBig->addAction(ui->actionGreekBig_theta);
    toolGreekBig->addAction(ui->actionGreekBig_upsilon);
    toolGreekBig->addAction(ui->actionGreekBig_xi);

    toolSums = new WidgetToolMenu( 3, 1, this);
    toolSums->addAction(ui->actionSum_sum);
    toolSums->addAction(ui->actionSum_sumArgs);
    toolSums->addAction(ui->actionSum_sumArgs2);
    
    toolIntegrals = new WidgetToolMenu(2,1, this);
    toolIntegrals->addAction(ui->actionInt_simple);
    toolIntegrals->addAction(ui->actionInt_oint);
    toolIntegrals->addAction(ui->actionInt_intDouble);
    toolIntegrals->addAction(ui->actionInt_oint2);
    toolIntegrals->addAction(ui->actionInt_int2);
    toolIntegrals->addAction(ui->actionInt_intTriple);
    toolIntegrals->addAction(ui->actionInt_intQuadruple);
    toolIntegrals->addAction(ui->actionInt_ointop);
    toolIntegrals->addAction(ui->actionInt_dots);

    toolProd = new WidgetToolMenu(2, 1, this);
    toolProd->addAction(ui->actionProd_prod);
    toolProd->addAction(ui->actionProd_coproduct);
    toolProd->addAction(ui->actionProd_prodArgs);
    toolProd->addAction(ui->actionProd_coproductArgs);
    toolProd->addAction(ui->actionProd_prodArgs2);
    toolProd->addAction(ui->actionProd_coproductArgs2);

    toolArrows = new WidgetToolMenu(6, 1, this);
    toolArrows->addAction(ui->actionArrow_rightarrow);
    toolArrows->addAction(ui->actionArrow_leftarrow);
    toolArrows->addAction(ui->actionArrow_downarrow);
    toolArrows->addAction(ui->actionArrow_uparrow);
    toolArrows->addAction(ui->actionArrow_updownarrow);
    toolArrows->addAction(ui->actionArrow_leftrightarrow);
    toolArrows->addAction(ui->actionArrow_leftrightarrow2);
    toolArrows->addAction(ui->actionArrow_leftarrow2);
    toolArrows->addAction(ui->actionArrow_rightarrow2);
    toolArrows->addAction(ui->actionArrow_downarrow2);
    toolArrows->addAction(ui->actionArrow_uparrow2);
    toolArrows->addAction(ui->actionArrow_updownarrow2);
    toolArrows->addAction(ui->actionArrow_longleftrightarrow2);
    toolArrows->addAction(ui->actionArrow_longleftarrow2);
    toolArrows->addAction(ui->actionArrow_longleftrightarrow2);
    toolArrows->addAction(ui->actionArrow_longleftrightarrow);
    toolArrows->addAction(ui->actionArrow_longleftarrow);
    toolArrows->addAction(ui->actionArrow_longleftrightarrow);
    toolArrows->addAction(ui->actionArrow_leftharpoondown);
    toolArrows->addAction(ui->actionArrow_rightharpoondown);
    toolArrows->addAction(ui->actionArrow_mapsto);
    toolArrows->addAction(ui->actionArrow_mapsto2);
    toolArrows->addAction(ui->actionArrow_arrowNW);
    toolArrows->addAction(ui->actionArrow_arrowNE);
    toolArrows->addAction(ui->actionArrow_leftharpoonup);
    toolArrows->addAction(ui->actionArrow_rightharpoonup);
    toolArrows->addAction(ui->actionArrow_hookleftarrow);
    toolArrows->addAction(ui->actionArrow_hookrightarrow);
    toolArrows->addAction(ui->actionArrow_arrowSW);
    toolArrows->addAction(ui->actionArrow_arrowSE);
    toolArrows->addAction(ui->actionArrow_rightleftharpoons);


}

void MainWindow::setMathPanel()
{

    //first row
    ui->mathPanelLayout->addWidget(toolDeco, 0 , 0);
    ui->mathPanelLayout->addWidget(toolFunc, 0, 1);
    ui->mathPanelLayout->addWidget(toolDelim, 0, 2);
    ui->mathPanelLayout->addWidget(toolAccents, 0, 3);
    ui->mathPanelLayout->addWidget(toolOps, 0, 4);
    ui->mathPanelLayout->addWidget(toolRelations, 0, 5);
    ui->mathPanelLayout->addWidget(toolMisc, 0, 6);
    ui->mathPanelLayout->addWidget(toolSets, 0, 7);

    // second row
    ui->mathPanelLayout->addWidget(toolGreek, 1, 0);
    ui->mathPanelLayout->addWidget(toolGreekBig, 1, 1);
    ui->mathPanelLayout->addWidget(toolSums, 1, 2);
    ui->mathPanelLayout->addWidget(toolIntegrals, 1, 3);
    ui->mathPanelLayout->addWidget(toolProd, 1, 4);
    ui->mathPanelLayout->addWidget(toolArrows, 1, 5, 1, 2);

}

void MainWindow::addButtonToMathTabs(const QString &eqImage, QWidget *tabWidget)
{
    QImage *eqImg = new QImage(eqImage);

    QAction *action = new QAction(tabWidget);
    action->setIcon(QIcon(eqImage) );
    action->setText(eqImg->text(INFO_HEADER_LATEX) );

    QToolButton *bt = new QToolButton;
    bt->setSizePolicy(QSizePolicy::Fixed, QSizePolicy::Expanding);
    bt->setDefaultAction(action);
    bt->setIconSize( eqImg->size() );

    connect(bt, SIGNAL(triggered(QAction*)), this, SLOT(insertSymbol(QAction*)) );

    tabWidget->layout()->addWidget(bt);
}

void MainWindow::addButtonToMathTabs(const QStringList &list, QWidget *tabWidget)
{

    for(int i = 0; i < list.size(); ++i){
        const QString eqFileImage = list.at(i);

        QImage *eqImg = new QImage(eqFileImage);

        QAction *action = new QAction(tabWidget);
        action->setIcon(QIcon(eqFileImage) );
        action->setText(eqImg->text(INFO_HEADER_LATEX) );

        QToolButton *bt = new QToolButton;
        bt->setSizePolicy(QSizePolicy::Fixed, QSizePolicy::Expanding);
        bt->setDefaultAction(action);
        bt->setIconSize( eqImg->size() );

        connect(bt, SIGNAL(triggered(QAction*)), this, SLOT(insertSymbol(QAction*)) );

        tabWidget->layout()->addWidget(bt);
    }

}

void MainWindow::setMathPanelTabsActions()
{
    // Algebra Tab
    QStringList listAlgebra;
    listAlgebra << ":/resources/icons/tabBar/algebra/eq01.png"
            << ":/resources/icons/tabBar/algebra/eq02.png"
            << ":/resources/icons/tabBar/algebra/eq03.png"
            << ":/resources/icons/tabBar/algebra/eq04.png"
            << ":/resources/icons/tabBar/algebra/eq05.png"
            << ":/resources/icons/tabBar/algebra/eq06.png" ;

    addButtonToMathTabs(listAlgebra, ui->tabMath);
    ((QHBoxLayout*)ui->tabMath->layout())->addStretch(0);

    // Derivs Tab
    QStringList listDerivs;
    listDerivs << ":/resources/icons/tabBar/derivs/eq01.png"
            << ":/resources/icons/tabBar/derivs/eq02.png"
            << ":/resources/icons/tabBar/derivs/eq03.png"
            << ":/resources/icons/tabBar/derivs/eq04.png"
            << ":/resources/icons/tabBar/derivs/eq05.png"
            << ":/resources/icons/tabBar/derivs/eq06.png" ;

    addButtonToMathTabs(listDerivs, ui->tabMath_2);
    ((QHBoxLayout*)ui->tabMath_2->layout())->addStretch(0);

    // Stats Tab
    QStringList listStats;
    listStats << ":/resources/icons/tabBar/stats/eq01.png"
            << ":/resources/icons/tabBar/stats/eq02.png"
            << ":/resources/icons/tabBar/stats/eq03.png"
            << ":/resources/icons/tabBar/stats/eq04.png"
            << ":/resources/icons/tabBar/stats/eq05.png"
            << ":/resources/icons/tabBar/stats/eq06.png"
            << ":/resources/icons/tabBar/stats/eq07.png" ;

    addButtonToMathTabs(listStats, ui->tabMath_3);
    ((QHBoxLayout*)ui->tabMath_3->layout())->addStretch(0);

    // Stats Tab
    QStringList listMatrix;
    listMatrix << ":/resources/icons/tabBar/matrix/eq01.png"
            << ":/resources/icons/tabBar/matrix/eq02.png"
            << ":/resources/icons/tabBar/matrix/eq03.png"
            << ":/resources/icons/tabBar/matrix/eq04.png";

    addButtonToMathTabs(listMatrix, ui->tabMath_4);
    ((QHBoxLayout*)ui->tabMath_4->layout())->addStretch(0);

    // Sets Tab
    QStringList listSets;
    listSets << ":/resources/icons/tabBar/sets/eq01.png"
            << ":/resources/icons/tabBar/sets/eq02.png";


    addButtonToMathTabs(listSets, ui->tabMath_5);
    ((QHBoxLayout*)ui->tabMath_5->layout())->addStretch(0);

    // Trig Tab
    QStringList listTrig;
    listTrig << ":/resources/icons/tabBar/trig/eq01.png"
            << ":/resources/icons/tabBar/trig/eq02.png"
            << ":/resources/icons/tabBar/trig/eq03.png"
            << ":/resources/icons/tabBar/trig/eq04.png";

    addButtonToMathTabs(listTrig, ui->tabMath_6);
    ((QHBoxLayout*)ui->tabMath_6->layout())->addStretch(0);

    // Geometry Tab
    QStringList listGeometry;
    listGeometry << ":/resources/icons/tabBar/geometry/eq01.png"
            << ":/resources/icons/tabBar/geometry/eq02.png"
            << ":/resources/icons/tabBar/geometry/eq03.png"
            << ":/resources/icons/tabBar/geometry/eq04.png";

    addButtonToMathTabs(listGeometry, ui->tabMath_7);
    ((QHBoxLayout*)ui->tabMath_7->layout())->addStretch(0);

    // Chemistry Tab
    QStringList listChemistry;
    listChemistry << ":/resources/icons/tabBar/chemistry/eq01.png"
            << ":/resources/icons/tabBar/chemistry/eq02.png"
            << ":/resources/icons/tabBar/chemistry/eq03.png"
            << ":/resources/icons/tabBar/chemistry/eq04.png"
            << ":/resources/icons/tabBar/chemistry/eq05.png";

    addButtonToMathTabs(listChemistry, ui->tabMath_8);
    ((QHBoxLayout*)ui->tabMath_8->layout())->addStretch(0);


    // Physics Tab
    QStringList listPhysics;
    listPhysics << ":/resources/icons/tabBar/physics/eq01.png"
            << ":/resources/icons/tabBar/physics/eq02.png"
            << ":/resources/icons/tabBar/physics/eq03.png"
            << ":/resources/icons/tabBar/physics/eq04.png";

    addButtonToMathTabs(listPhysics, ui->tabMath_9);
    ((QHBoxLayout*)ui->tabMath_9->layout())->addStretch(0);



}

void MainWindow::setSignals()
{
    /* MainWindow */
    // Menu Bar
    // File Menu
    connect(ui->actionNew, SIGNAL(triggered()), this, SLOT(newEquation()) );
    connect(ui->actionOpen, SIGNAL(triggered()), this, SLOT(open()) );
    connect(ui->actionSave, SIGNAL(triggered()), this, SLOT(save()) );
    connect(ui->actionSaveAs, SIGNAL(triggered()), this, SLOT(saveAs()) );

    // Export To...
    connect(ui->actionExport_DVI, SIGNAL(triggered()), this, SLOT(exportToDVI()) );
    connect(ui->actionExport_TeX, SIGNAL(triggered()), this, SLOT(exportToTeX()) );
    connect(ui->actionExport_PDF, SIGNAL(triggered()), this, SLOT(exportToPDF()) );
    connect(ui->actionExport_PS, SIGNAL(triggered()), this, SLOT(exportToPS()) );
    connect(ui->actionExport_GIF, SIGNAL(triggered()), this, SLOT(exportToGIF()) );
    connect(ui->actionExport_PNG, SIGNAL(triggered()), this, SLOT(saveAs()) );
    connect(ui->actionExport_MathML, SIGNAL(triggered()), this, SLOT(exportToMathML()) );
    connect(ui->actionExport_SVG, SIGNAL(triggered()), this, SLOT(exportToSVG()) );

    connect(ui->actionQuit, SIGNAL(triggered()), this, SLOT(close()) );

    // Help Menu
    connect(ui->actionHelp_About, SIGNAL(triggered()), this, SLOT(about()) );
    connect(ui->actionHelp_AboutQt, SIGNAL(triggered()), qApp, SLOT(aboutQt()) );



    // GUI
    connect(editor, SIGNAL(textChanged()), this, SLOT(editorChanged()) );
    connect(ui->editPreamble, SIGNAL(textChanged()), this, SLOT(editorChanged()) );
    connect(ui->fontSizeComboBox, SIGNAL(currentIndexChanged(int)) , this, SLOT(resolutionChanged(int)) );
    connect(ui->fgColorPicker, SIGNAL(colorSelectionChanged(QColor)), this, SLOT(fgColorChanged(QColor)) );
    connect(ui->bgColorPicker, SIGNAL(colorSelectionChanged(QColor)), this, SLOT(bgColorChanged(QColor)) );
    connect(ui->inlineBox, SIGNAL(toggled(bool)), this, SLOT(modeChanged(bool)) );

    // Misc
    connect(&timer, SIGNAL(timeout()), this, SLOT(updateEquation()) );

    //for MathPanel
    connect(toolDeco, SIGNAL(triggered(QAction*)), this, SLOT(insertSymbol(QAction*)) );
    connect(toolFunc, SIGNAL(triggered(QAction*)), this, SLOT(insertSymbol(QAction*)) );
    connect(toolDelim, SIGNAL(triggered(QAction*)), this, SLOT(insertSymbol(QAction*)) );
    connect(toolAccents, SIGNAL(triggered(QAction*)), this, SLOT(insertSymbol(QAction*)) );
    connect(toolOps, SIGNAL(triggered(QAction*)), this, SLOT(insertSymbol(QAction*)) );
    connect(toolRelations, SIGNAL(triggered(QAction*)), this, SLOT(insertSymbol(QAction*)) );
    connect(toolMisc, SIGNAL(triggered(QAction*)), this, SLOT(insertSymbol(QAction*)) );
    connect(toolSets, SIGNAL(triggered(QAction*)), this, SLOT(insertSymbol(QAction*)) );
    connect(toolGreek, SIGNAL(triggered(QAction*)), this, SLOT(insertSymbol(QAction*)) );
    connect(toolGreekBig, SIGNAL(triggered(QAction*)), this, SLOT(insertSymbol(QAction*)) );
    connect(toolSums, SIGNAL(triggered(QAction*)), this, SLOT(insertSymbol(QAction*)) );
    connect(toolIntegrals, SIGNAL(triggered(QAction*)), this, SLOT(insertSymbol(QAction*)) );
    connect(toolProd, SIGNAL(triggered(QAction*)), this, SLOT(insertSymbol(QAction*)) );
    connect(toolArrows, SIGNAL(triggered(QAction*)), this, SLOT(insertSymbol(QAction*)) );

}


void MainWindow::closeEvent(QCloseEvent *event)
{
    if ( wishToSave() ) {
        writeSettings();
        event->accept();
    } else {
        event->ignore();
    }
}

bool MainWindow::exportToDVI()
{
    QString fileName = saveTo("DVI File (*.dvi)");

    if(fileName.isEmpty())
        return false;
    qDebug() << "Export to DVI " << fileName;

    QFile tempFile(QString("%1.dvi").arg(TEMP_FILE) );
    return tempFile.copy(fileName);

}

bool MainWindow::exportToTeX(){}
bool MainWindow::exportToPDF(){}
bool MainWindow::exportToPS(){}
bool MainWindow::exportToGIF(){}
bool MainWindow::exportToMathML(){}
bool MainWindow::exportToSVG(){}

void MainWindow::loadTexTemplate()
{
    QFile file(":/resources/templates/template-content.tex");
    if (!file.open(QIODevice::ReadOnly | QIODevice::Text))
        return;

    QTextStream in(&file);
    texTemplateContent = in.readAll();

    QFile file2(":/resources/templates/template-preamble.tex");
    if (!file2.open(QIODevice::ReadOnly | QIODevice::Text))
        return;

    QTextStream in2(&file2);
    QString texTemplatePreamble = in2.readAll();

    ui->editPreamble->setText(texTemplatePreamble);

}

void MainWindow::newEquation()
{
    if (wishToSave()) {
        editor->clear();
        setCurrentFile("");
    }

}

void MainWindow::open()
{
    QString curDirToSave;

    if (wishToSave()) {

        if( curFile.isEmpty() )
            curDirToSave = QDir::homePath();
        else
            curDirToSave = QFileInfo(curFile).dir().path();

        QString fileName = QFileDialog::getOpenFileName(this, tr("Open Equation"),
                                                        curDirToSave,
                                                        tr("Image Equation (*.png)"));
        if (!fileName.isEmpty())
            loadEquation(fileName);
    }
}

bool MainWindow::save()
{


    if (curFile.isEmpty()) {
        return saveAs();
    } else {
        return saveEquation(curFile);
    }
}

bool MainWindow::saveAs()
{
    QString fileName = saveTo("Equation Image (*.png)");

    if (fileName.isEmpty())
        return false;

    return saveEquation(fileName);
}

QString MainWindow::saveTo(const QString &filter)
{
    QString curDirToSave;

    if( curFile.isEmpty() )
        curDirToSave = QDir::homePath();
    else
        curDirToSave = QFileInfo(curFile).dir().path();

    QString fileName = QFileDialog::getSaveFileName(this, tr("Save Equation as..."), curDirToSave, filter );

    return fileName;
}

void MainWindow::about()
{

    QFile file(":/LICENSE");
    if (!file.open(QIODevice::ReadOnly | QIODevice::Text))
        return;

    QTextStream in(&file);
    QString licenseText = in.readAll();
    in.flush();

    QString aboutText = QString("%1 <br><b>Version %2</b> <br><br>%3<br> <pre>%4</pre> ").arg(APP_FULL_NAME).arg(APP_VERSION).arg("A LateX Equation Editor").arg(licenseText);


    QMessageBox::about(this, tr("About EqualX"), aboutText );
}

void MainWindow::writePNGMetaData(const QString &pngFile)
{
    QImage tmp_imgFile(pngFile);
    tmp_imgFile.setText(INFO_HEADER_GEN, APP_NAME);
    tmp_imgFile.setText(INFO_HEADER_GEN_VER,QString::number(APP_VERSION) );
    tmp_imgFile.setText(INFO_HEADER_LATEX, editor->toPlainText() ) ;
    tmp_imgFile.setText(INFO_HEADER_FG, ui->fgColorPicker->getColorName() );
    tmp_imgFile.setText(INFO_HEADER_BG, ui->bgColorPicker->getColorName() );
    tmp_imgFile.setText(INFO_HEADER_RES, QString::number(ui->fontSizeComboBox->currentIndex()) );
    tmp_imgFile.setText(INFO_HEADER_PREAMBLE, ui->editPreamble->toPlainText() );
    tmp_imgFile.save(pngFile);

}

void MainWindow::readSettings()
{
    QSettings settings(SETTINGS_DIR, SETTINGS_FILE);

    settings.beginGroup("MainWindow");
    resize(settings.value("size", QSize(APP_WIDTH, APP_HEIGHT)).toSize() );
    move(settings.value("position", QPoint(100, 100)).toPoint());
    settings.endGroup();

    settings.beginGroup("Extern Applications");
    settings.value("Latex2DVI", LATEX2DVI);
    settings.value("DVI2PNG", DVI2PNG);
    settings.endGroup();
    /*
    int size = settings.beginReadArray("Library");
    for (int i = 0; i < size; ++i) {
        settings.setArrayIndex(i);
        EquationItem equationItem;
        equationItem.thumb = settings.value("thumb").toString();
        equationItem.form  = settings.value("form").toString();

        addEquationToLibrary(equationItem);
    }
    settings.endArray();
*/
    settings.beginGroup("Paths");
    tempFileName = settings.value("TempTexFile", TEMP_FILE).toString();
    settings.endGroup();
}

void MainWindow::setupParser()
{
    parser.addToken("font");
    parser.addToken("foreground_color");
    parser.addToken("background_color");
    parser.addToken("displaymath", "show");
    parser.addToken("input");
}

void MainWindow::updateDisplay()
{
    equation->setPixmap( workspace.tempImage() );

    QRectF imageBorders = equation->boundingRect();

    scene->setSceneRect(imageBorders);

    equation->setPos(scene->width()/2 - imageBorders.width()/2, scene->height()/2 - imageBorders.height()/2);

}

void MainWindow::updateImage()
{
    int size;

    size = resolution * 9.636;

    QString cmdStrDVI2PNG = QString(DVI2PNG).arg( workspace.tempImage() ).arg(tempFileName).arg(size).arg(bgColor);
    QProcess process2;

    qDebug() << "\n\n[process]" << cmdStrDVI2PNG;

    process2.setProcessChannelMode(QProcess::MergedChannels);
    process2.setWorkingDirectory(workspace.tempDir());

    process2.start(cmdStrDVI2PNG);

    if (!process2.waitForFinished())
        qDebug() << "[Failed] " << process2.errorString();
    else
        qDebug() << "[output]" << process2.readAll();

    QString imgFileStr = workspace.tempImage();

    qDebug() << "METADATA FILE =" << imgFileStr;

    writePNGMetaData(imgFileStr);


}

void MainWindow::updateEquation()
{
    timer.stop();

    if(isDirty){
        qDebug() << "================================== < UPDATE > [ " << nUpdates << " ]==================================";

        QString latexEquation = editor->toPlainText();
        QString texTemplatePreamble = ui->editPreamble->toPlainText();

        // parse Tex template
        parser.setTokenValue("font", "sans");
        parser.setTokenValue("foreground_color", fgColor);

        if(isInline)
        {
            parser.setTokenValue("displaymath", "");
            latexEquation = "$"+latexEquation+"$";
        }
        else{
            parser.setTokenValue("displaymath", "show");
        }

        parser.setTokenValue("input", latexEquation);

        QString parsedTexContent = parser.parse(texTemplatePreamble+texTemplateContent);

        // write parsed Tex content to a temp file
        QFile tmpTexFile( workspace.tempFile().append(".tex") );

        if (!tmpTexFile.open(QIODevice::WriteOnly | QIODevice::Text))
            return;

        QTextStream out(&tmpTexFile);
        out << parsedTexContent;
        out.flush();


        // create commands
        QString cmdStrTex2DVI = QString(LATEX2DVI).arg(workspace.tempFile());

        QProcess process1;


        qDebug() << "\n\n[process]" << cmdStrTex2DVI;

        process1.setProcessChannelMode(QProcess::MergedChannels);
        process1.setWorkingDirectory(workspace.tempDir());

        process1.start(cmdStrTex2DVI);

        if (!process1.waitForFinished())
            qDebug() << "[Failed] " << process1.errorString();
        else
            qDebug() << "[output]" << process1.readAll();

        updateImage();
        updateDisplay();

        if(! equation->isVisible() )
            equation->setVisible(true);

        isDirty = false;
        nUpdates++;
    }
}

void MainWindow::writeSettings()
{
    QSettings settings(SETTINGS_DIR, SETTINGS_FILE);

    settings.beginGroup("MainWindow");
    settings.setValue("size", size() );
    settings.setValue("position", pos());
    settings.endGroup();

    settings.beginGroup("Extern Applications");
    settings.setValue("Latex2DVI", LATEX2DVI);
    settings.setValue("DVI2PNG", DVI2PNG);
    settings.endGroup();
    /*
    settings.beginWriteArray("Library");
    for (int i = 0; i < library.size(); ++i) {
        settings.setArrayIndex(i);
        settings.setValue("thumb", library.at(i).thumb);
        settings.setValue("form", library.at(i).form);
    }
    settings.endArray();
*/
    settings.beginGroup("Paths");
    settings.setValue("TempTexFile", tempFileName);
    settings.endGroup();
}


void MainWindow::editorChanged()
{
    setWindowModified(editor->document()->isModified());

    if( timer.isActive() )
        timer.stop();

    timer.setSingleShot(true);
    timer.start(EQUATION_UPDATE_INTERVAL);
    isDirty = true;

}

void MainWindow::resolutionChanged(const int res)
{
    if( timer.isActive() )
        timer.stop();

    resolution = ui->fontSizeComboBox->currentText().toInt();
    setWindowModified(true);



    timer.setSingleShot(true);
    timer.start(EQUATION_UPDATE_INTERVAL);
    isDirty = true;
}

void MainWindow::fgColorChanged(const QColor &color)
{
    if( timer.isActive() )
        timer.stop();

    if(color.alpha() > 0.0){
        fgColor = QString("%1,%2,%3").arg(color.redF()).arg(color.greenF()).arg(color.blueF());
    }
    else{
        fgColor = "Transparent";
    }
    setWindowModified(true);


    timer.setSingleShot(true);
    timer.start(EQUATION_UPDATE_INTERVAL);
    isDirty = true;
}

void MainWindow::bgColorChanged(const QColor &color)
{
    if( timer.isActive() )
        timer.stop();

    if(color.alpha() > 0.0){
        bgColor = QString("\"rgb %1 %2 %3\"").arg(color.redF()).arg(color.greenF()).arg(color.blueF());
    }
    else{
        bgColor = "Transparent";
    }

    setWindowModified(true);

    timer.setSingleShot(true);
    timer.start(EQUATION_UPDATE_INTERVAL);
    isDirty = true;
}

void MainWindow::modeChanged(bool mode)
{
    if( timer.isActive() )
        timer.stop();

    isInline = mode;
    setWindowModified(true);

    timer.setSingleShot(true);
    timer.start(EQUATION_UPDATE_INTERVAL);
    isDirty = true;
}

void MainWindow::insertSymbol(QAction *action)
{
    editor->insertPlainText(action->text() );

}

bool MainWindow::wishToSave()
{
    if ( isWindowModified () ) {
        QMessageBox::StandardButton ret;
        ret = QMessageBox::warning(this, APP_FULL_NAME,
                                   tr("The Equation has been modified.\n"
                                      "Do you want to save your changes?"),
                                   QMessageBox::Save | QMessageBox::Discard | QMessageBox::Cancel);
        if (ret == QMessageBox::Save)
            return save();
        else if (ret == QMessageBox::Cancel)
            return false;
    }

    return true;
}

void MainWindow::loadEquation(const QString &fileName)
{
    qDebug() << "Loading equation " << fileName;
    QImage m_image(fileName);

    if(m_image.isNull())
        return;

    // check if file was created using EqualX
    if( !(m_image.text(INFO_HEADER_GEN) == APP_NAME) ){
        QMessageBox::StandardButton a = QMessageBox::warning(this, APP_FULL_NAME,
                                                             QString("This file was not created using %1.\n Do you want to try to load it anyway?").arg(APP_FULL_NAME),
                                                             QMessageBox::Ok | QMessageBox::Cancel);

        if( a == QMessageBox::Cancel ) return;


    }

    // check if file was created using a recent version of Equalx
    if( (m_image.text(INFO_HEADER_GEN_VER).toDouble() > APP_VERSION)  && (m_image.text(INFO_HEADER_GEN) == APP_NAME) ){
        QMessageBox::StandardButton a = QMessageBox::warning(this, APP_FULL_NAME,
                                                             QString("This equation was created using a newer version of %1.\n Some features might not work. Do you want to try to load it anyway?").arg(APP_FULL_NAME),
                                                             QMessageBox::Ok | QMessageBox::Cancel);

        if( a == QMessageBox::Cancel ) return;


    }

    // check if file has latex information
    QString latexString = m_image.text(INFO_HEADER_LATEX);
    if(latexString.isEmpty() ){
        QMessageBox::information(this, APP_FULL_NAME,
                                 QString("This equation does not contain any latex information.\n %1 can not load it.").arg(APP_FULL_NAME),
                                 QMessageBox::Ok );

        statusBar()->showMessage(tr("File not loaded"), 2000);
        return;

    }

    // set widgets data
    editor->setPlainText(latexString);

    if( !m_image.text(INFO_HEADER_PREAMBLE).isEmpty() )
        ui->editPreamble->setPlainText(m_image.text(INFO_HEADER_PREAMBLE) );

    ui->fgColorPicker->setColor(m_image.text(INFO_HEADER_FG) );
    ui->bgColorPicker->setColor(m_image.text(INFO_HEADER_BG) );
    ui->fontSizeComboBox->setCurrentIndex( m_image.text(INFO_HEADER_RES).toInt() );

    setCurrentFile(fileName);
    statusBar()->showMessage( QString("File loaded: %1").arg(fileName), 5000);
}

bool MainWindow::saveEquation(const QString &fileName)
{
    equation->pixmap().save(fileName, "PNG");
    writePNGMetaData(fileName);

    setCurrentFile(fileName);
    statusBar()->showMessage(tr("File saved"), 5000);
    return true;
}

void MainWindow::setCurrentFile(const QString &fileName)
{
    curFile = fileName;
    editor->document()->setModified(false);
    setWindowModified(false);

    QString shownName;
    if (curFile.isEmpty())
        shownName = "untitled.png";
    else
        shownName = strippedName(curFile);

    setWindowTitle(tr("%1[*] - %2").arg(shownName).arg(APP_FULL_NAME));
}

QString MainWindow::strippedName(const QString &fullFileName)
{
    return QFileInfo(fullFileName).fileName();
}
